<?php

namespace HubPublic\Serializers;

use HubPublic\Collections\CartItemCollection;
use HubPublic\ValueObjects\CartContent;
use HubPublic\ValueObjects\CartItem;
use HubPublic\Exceptions\BadSerializerValueException;
use HubPublic\Serializers\Interfaces\SerializerInterface;
/**
 * Class CartContentSerializer
 *
 * @package HubPublic\Serializers
 */
class CartContentSerializer implements SerializerInterface
{
    /**
     * Contains the cart item names as key and the type condition function as value.
     *
     * @var array
     */
    private static $cartItemTypes = ['ean' => 'is_string', 'name' => 'is_string', 'price' => 'is_numeric', 'quantity' => 'is_numeric'];
    /**
     * Deserialize a JSON string.
     *
     * @param string $string JSON string that contains the data.
     *
     * @return \HubPublic\ValueObjects\CartContent CartContent instance that contains the deserialized data
     * @throws \HubPublic\Exceptions\BadSerializerValueException If JSON string is malformed.
     */
    public function deserialize($string)
    {
        if (empty($string)) {
            throw new BadSerializerValueException('Given string is empty.');
        }
        $json = json_decode($string, true);
        // error for malformed json strings
        if ($json === null && json_last_error() > 0) {
            throw new BadSerializerValueException('Provided JSON string is malformed and could not be parsed: ' . $string);
        }
        $cartItemCollection = $this->_getCartItemCollection($json);
        return new CartContent($cartItemCollection, (double) $json['totalPrice']);
    }
    /**
     * Serialize a value to a JSON string or array.
     *
     * @param \HubPublic\ValueObjects\CartContent $cartContent Content to be serialized.
     * @param bool                                $encode      Serialize to string?
     *
     * @return array|string Serialized JSON string or array of given content.
     *
     * @throws \HubPublic\Exceptions\BadSerializerValueException If passed argument is not a CartContent.
     */
    public function serialize($cartContent, $encode = true)
    {
        if (!is_object($cartContent) || !$cartContent instanceof CartContent) {
            throw new BadSerializerValueException('Argument is not a CartContent: ' . gettype($cartContent));
        }
        $json = ['cartItems' => $this->_serializeCartItems($cartContent->getCartItemCollection()), 'totalPrice' => $cartContent->getTotalPrice()];
        return $encode ? json_encode($json, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES) : $json;
    }
    /**
     * Create a CartItemCollection from json cart item data.
     *
     * @param array $json Array of cart items
     *
     * @return \HubPublic\Collections\CartItemCollection CartItemCollection instance that contains the deserialized data
     * @throws \HubPublic\Exceptions\BadSerializerValueException If cart item information are invalid
     */
    private function _getCartItemCollection(array $json)
    {
        $cartItemCollection = new CartItemCollection();
        foreach ($json['cartItems'] as $cartItem) {
            $ean = $cartItem['ean'];
            $name = $cartItem['name'];
            $price = (double) $cartItem['price'];
            $quantity = (double) $cartItem['quantity'];
            $attributes = [];
            if (array_key_exists('attributes', $cartItem)) {
                $attributes = $cartItem['attributes'];
            }
            $cartItemCollection->add(new CartItem($ean, $name, $price, $quantity, $attributes));
        }
        return $cartItemCollection;
    }
    /**
     * Serialize the CartItem collection.
     *
     * @param \HubPublic\Collections\CartItemCollection $collection
     *
     * @return array
     */
    private function _serializeCartItems(CartItemCollection $collection)
    {
        $json = [];
        /** @var CartItem $cartItem */
        foreach ($collection->asArray() as $cartItem) {
            $json[] = ['ean' => $cartItem->getEan(), 'name' => $cartItem->getName(), 'price' => $cartItem->getPrice(), 'quantity' => $cartItem->getQuantity(), 'attributes' => $cartItem->getAttributes()];
        }
        return $json;
    }
}